@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright (C) 2013-2022 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -70,6 +70,7 @@
 #include <pthread_time.h>
 #endif
 
+#include "array-size.h"
 #include "ascii-ctype.h"
 #include "ascii-string.h"
 #include "get_current_dir_name.h"
@@ -82,7 +83,7 @@
 
 #ifndef WIN32
 
-char *
+NBDKIT_DLL_PUBLIC char *
 nbdkit_absolute_path (const char *path)
 {
   CLEANUP_FREE char *pwd = NULL;
@@ -123,7 +124,7 @@ nbdkit_absolute_path (const char *path)
  * replace nbdkit_absolute_path with nbdkit_realpath and everything
  * should work the same.
  */
-char *
+NBDKIT_DLL_PUBLIC char *
 nbdkit_absolute_path (const char *path)
 {
   return nbdkit_realpath (path);
@@ -131,7 +132,7 @@ nbdkit_absolute_path (const char *path)
 
 #endif /* WIN32 */
 
-char *
+NBDKIT_DLL_PUBLIC char *
 nbdkit_realpath (const char *path)
 {
   char *ret;
@@ -173,7 +174,7 @@ nbdkit_realpath (const char *path)
   return 0
 
 /* Functions for parsing signed integers. */
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_int (const char *what, const char *str, int *rp)
 {
   long r;
@@ -188,7 +189,7 @@ nbdkit_parse_int (const char *what, const char *str, int *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_int8_t (const char *what, const char *str, int8_t *rp)
 {
   long r;
@@ -201,7 +202,7 @@ nbdkit_parse_int8_t (const char *what, const char *str, int8_t *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_int16_t (const char *what, const char *str, int16_t *rp)
 {
   long r;
@@ -214,7 +215,7 @@ nbdkit_parse_int16_t (const char *what, const char *str, int16_t *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_int32_t (const char *what, const char *str, int32_t *rp)
 {
   long r;
@@ -229,7 +230,7 @@ nbdkit_parse_int32_t (const char *what, const char *str, int32_t *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_int64_t (const char *what, const char *str, int64_t *rp)
 {
   long long r;
@@ -259,7 +260,7 @@ nbdkit_parse_int64_t (const char *what, const char *str, int64_t *rp)
     }                                                                   \
   } while (0)
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_unsigned (const char *what, const char *str, unsigned *rp)
 {
   unsigned long r;
@@ -275,7 +276,7 @@ nbdkit_parse_unsigned (const char *what, const char *str, unsigned *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_uint8_t (const char *what, const char *str, uint8_t *rp)
 {
   unsigned long r;
@@ -289,7 +290,7 @@ nbdkit_parse_uint8_t (const char *what, const char *str, uint8_t *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_uint16_t (const char *what, const char *str, uint16_t *rp)
 {
   unsigned long r;
@@ -303,7 +304,7 @@ nbdkit_parse_uint16_t (const char *what, const char *str, uint16_t *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_uint32_t (const char *what, const char *str, uint32_t *rp)
 {
   unsigned long r;
@@ -319,7 +320,7 @@ nbdkit_parse_uint32_t (const char *what, const char *str, uint32_t *rp)
   PARSE_COMMON_TAIL;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_uint64_t (const char *what, const char *str, uint64_t *rp)
 {
   unsigned long long r;
@@ -338,7 +339,7 @@ nbdkit_parse_uint64_t (const char *what, const char *str, uint64_t *rp)
 /* Parse a string as a size with possible scaling suffix, or return -1
  * after reporting the error.
  */
-int64_t
+NBDKIT_DLL_PUBLIC int64_t
 nbdkit_parse_size (const char *str)
 {
   int64_t size;
@@ -422,7 +423,7 @@ nbdkit_parse_size (const char *str)
 
 /* Parse a string as a boolean, or return -1 after reporting the error.
  */
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_parse_bool (const char *str)
 {
   if (!strcmp (str, "1") ||
@@ -446,7 +447,7 @@ nbdkit_parse_bool (const char *str)
 }
 
 /* Return true if it is safe to read from stdin during configuration. */
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_stdio_safe (void)
 {
   return !listen_stdin && !configured;
@@ -456,7 +457,7 @@ nbdkit_stdio_safe (void)
 static int read_password_interactive (char **password);
 static int read_password_from_fd (const char *what, int fd, char **password);
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_read_password (const char *value, char **password)
 {
   *password = NULL;
@@ -674,7 +675,7 @@ read_password_from_fd (const char *what, int fd, char **password)
   return 0;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_nanosleep (unsigned sec, unsigned nsec)
 {
   struct timespec ts;
@@ -693,6 +694,8 @@ nbdkit_nanosleep (unsigned sec, unsigned nsec)
    * - the current connection is multi-threaded and another thread detects
    *   NBD_CMD_DISC or a problem with the connection
    * - the input socket detects POLLRDHUP/POLLHUP/POLLERR
+   * - the input socket is invalid (POLLNVAL, probably closed by
+   *   another thread)
    */
   struct connection *conn = threadlocal_get_conn ();
   struct pollfd fds[] = {
@@ -710,7 +713,7 @@ nbdkit_nanosleep (unsigned sec, unsigned nsec)
    */
   if (sigfillset(&all))
     abort ();
-  switch (ppoll (fds, sizeof fds / sizeof fds[0], &ts, &all)) {
+  switch (ppoll (fds, ARRAY_SIZE (fds), &ts, &all)) {
   case -1:
     assert (errno != EINTR);
     nbdkit_error ("poll: %m");
@@ -722,10 +725,15 @@ nbdkit_nanosleep (unsigned sec, unsigned nsec)
   /* We don't have to read the pipe-to-self; if poll returned an
    * event, we know the connection should be shutting down.
    */
-  assert (quit ||
+  bool has_quit = quit;
+  assert (has_quit ||
           (conn && conn->nworkers > 0 && connection_get_status () < 1) ||
-          (conn && (fds[2].revents & (POLLRDHUP | POLLHUP | POLLERR))));
-  nbdkit_error ("aborting sleep to shut down");
+          (conn && (fds[2].revents & (POLLRDHUP | POLLHUP | POLLERR |
+                                      POLLNVAL))));
+  if (has_quit)
+    nbdkit_error ("aborting sleep because of server shut down");
+  else
+    nbdkit_error ("aborting sleep because of connection close or error");
   errno = ESHUTDOWN;
   return -1;
 
@@ -756,36 +764,43 @@ nbdkit_nanosleep (unsigned sec, unsigned nsec)
 /* This function will be deprecated for API V3 users.  The preferred
  * approach will be to get the exportname from .open().
  */
-const char *
+NBDKIT_DLL_PUBLIC const char *
 nbdkit_export_name (void)
 {
-  struct connection *conn = threadlocal_get_conn ();
+  struct context *c = threadlocal_get_context ();
 
-  if (!conn) {
+  if (!c || !c->conn) {
     nbdkit_error ("no connection in this thread");
     return NULL;
   }
 
-  return conn->exportname;
+  return c->conn->exportname;
 }
 
 /* This function will be deprecated for API V3 users.  The preferred
  * approach will be to get the tls mode from .open().
  */
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_is_tls (void)
 {
-  struct connection *conn = threadlocal_get_conn ();
+  struct context *c = threadlocal_get_context ();
 
-  if (!conn) {
+  if (!c) {
     nbdkit_error ("no connection in this thread");
     return -1;
   }
 
-  return conn->using_tls;
+  if (!c->conn) {
+    /* If a filter opened this backend outside of a client connection,
+     * then we can only claim tls when the command line required it.
+     */
+    return tls == 2;
+  }
+
+  return c->conn->using_tls;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_peer_name (struct sockaddr *addr, socklen_t *addrlen)
 {
   struct connection *conn = threadlocal_get_conn ();
@@ -810,36 +825,50 @@ nbdkit_peer_name (struct sockaddr *addr, socklen_t *addrlen)
   return 0;
 }
 
-#ifdef SO_PEERCRED
+#if defined(SO_PEERCRED) && \
+  (defined(HAVE_STRUCT_UCRED_UID) || defined(HAVE_STRUCT_SOCKPEERCRED_UID))
+
+#define GET_PEERCRED_DEFINED 1
 
 static int
 get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
 {
-  struct ucred ucred;
-  socklen_t n = sizeof ucred;
+#if HAVE_STRUCT_UCRED_UID
+  struct ucred cred;
+#elif HAVE_STRUCT_SOCKPEERCRED_UID
+  /* The struct has a different name on OpenBSD, but the same members. */
+  struct sockpeercred cred;
+#endif
+  socklen_t n = sizeof cred;
 
-  if (getsockopt (s, SOL_SOCKET, SO_PEERCRED, &ucred, &n) == -1) {
+  if (getsockopt (s, SOL_SOCKET, SO_PEERCRED, &cred, &n) == -1) {
     nbdkit_error ("getsockopt: SO_PEERCRED: %m");
     return -1;
   }
 
-  if (pid && ucred.pid >= 1) {
-    if (ucred.pid <= INT64_MAX)
-      *pid = ucred.pid;
-    else
+  if (pid && cred.pid >= 1) {
+#if SIZEOF_PID_T >= 8
+    if (cred.pid > INT64_MAX)
       nbdkit_error ("pid out of range: cannot be mapped to int64_t");
-  }
-  if (uid && ucred.uid >= 0) {
-    if (ucred.uid <= INT64_MAX)
-      *uid = ucred.uid;
     else
+#endif
+      *pid = cred.pid;
+  }
+  if (uid && cred.uid >= 0) {
+#if SIZEOF_UID_T >= 8
+    if (cred.uid > INT64_MAX)
       nbdkit_error ("uid out of range: cannot be mapped to int64_t");
-  }
-  if (gid && ucred.gid >= 0) {
-    if (ucred.gid <= INT64_MAX)
-      *gid = ucred.gid;
     else
+#endif
+      *uid = cred.uid;
+  }
+  if (gid && cred.gid >= 0) {
+#if SIZEOF_GID_T >= 8
+    if (cred.gid > INT64_MAX)
       nbdkit_error ("gid out of range: cannot be mapped to int64_t");
+    else
+#endif
+      *gid = cred.gid;
   }
 
   return 0;
@@ -849,6 +878,8 @@ get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
 
 #ifdef LOCAL_PEERCRED
 
+#define GET_PEERCRED_DEFINED 1
+
 /* FreeBSD supports LOCAL_PEERCRED and struct xucred. */
 static int
 get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
@@ -877,16 +908,24 @@ get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
   if (pid)
     nbdkit_error ("nbdkit_peer_pid is not supported on this platform");
   if (uid && xucred.cr_uid >= 0) {
+#if SIZEOF_UID_T >= 8
     if (xucred.cr_uid <= INT64_MAX)
+#endif
       *uid = xucred.cr_uid;
+#if SIZEOF_UID_T >= 8
     else
       nbdkit_error ("uid out of range: cannot be mapped to int64_t");
+#endif
   }
   if (gid && xucred.cr_ngroups > 0) {
+#if SIZEOF_GID_T >= 8
     if (xucred.cr_gid <= INT64_MAX)
+#endif
       *gid = xucred.cr_gid;
+#if SIZEOF_GID_T >= 8
     else
       nbdkit_error ("gid out of range: cannot be mapped to int64_t");
+#endif
   }
 
   return 0;
@@ -894,7 +933,7 @@ get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
 
 #endif /* LOCAL_PEERCRED */
 
-#if !defined(SO_PEERCRED) && !defined(LOCAL_PEERCRED)
+#ifndef GET_PEERCRED_DEFINED
 
 static int
 get_peercred (int s, int64_t *pid, int64_t *uid, int64_t *gid)
@@ -930,7 +969,7 @@ get_peercred_common (int64_t *pid, int64_t *uid, int64_t *gid)
   return get_peercred (s, pid, uid, gid);
 }
 
-int64_t
+NBDKIT_DLL_PUBLIC int64_t
 nbdkit_peer_pid ()
 {
   int64_t pid;
@@ -941,7 +980,7 @@ nbdkit_peer_pid ()
   return pid;
 }
 
-int64_t
+NBDKIT_DLL_PUBLIC int64_t
 nbdkit_peer_uid ()
 {
   int64_t uid;
@@ -952,7 +991,7 @@ nbdkit_peer_uid ()
   return uid;
 }
 
-int64_t
+NBDKIT_DLL_PUBLIC int64_t
 nbdkit_peer_gid ()
 {
   int64_t gid;
@@ -980,7 +1019,8 @@ free_interns (void)
 static const char *
 add_intern (char *str)
 {
-  struct connection *conn = threadlocal_get_conn ();
+  struct context *c = threadlocal_get_context ();
+  struct connection *conn = c ? c->conn : NULL;
   string_vector *list = conn ? &conn->interns : &global_interns;
 
   if (string_vector_append (list, str) == -1) {
@@ -992,7 +1032,7 @@ add_intern (char *str)
   return str;
 }
 
-const char *
+NBDKIT_DLL_PUBLIC const char *
 nbdkit_strndup_intern (const char *str, size_t n)
 {
   char *copy;
@@ -1012,7 +1052,7 @@ nbdkit_strndup_intern (const char *str, size_t n)
   return add_intern (copy);
 }
 
-const char *
+NBDKIT_DLL_PUBLIC const char *
 nbdkit_strdup_intern (const char *str)
 {
   char *copy;
@@ -1032,7 +1072,7 @@ nbdkit_strdup_intern (const char *str)
   return add_intern (copy);
 }
 
-const char *
+NBDKIT_DLL_PUBLIC const char *
 nbdkit_vprintf_intern (const char *fmt, va_list ap)
 {
   char *str = NULL;
@@ -1045,7 +1085,7 @@ nbdkit_vprintf_intern (const char *fmt, va_list ap)
   return add_intern (str);
 }
 
-const char *
+NBDKIT_DLL_PUBLIC const char *
 nbdkit_printf_intern (const char *fmt, ...)
 {
   va_list ap;
