@@ -38,6 +38,7 @@ requires_plugin linuxdisk
 requires guestfish --version
 requires nbdcopy --version
 requires qemu-img --version
+requires $STAT --version
 
 sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="cow-base.img cow-diff.qcow2 $sock cow.pid"
@@ -50,7 +51,7 @@ mkdir cow.d
 cleanup_fn rm -rf cow.d
 nbdkit -fv -U - linuxdisk cow.d size=100M \
        --run 'nbdcopy "$uri" cow-base.img'
-lastmod="$(stat -c "%y" cow-base.img)"
+lastmod="$($STAT -c "%y" cow-base.img)"
 
 # Run nbdkit with a COW overlay.
 start_nbdkit -P cow.pid -U $sock --filter=cow file cow-base.img
@@ -62,7 +63,7 @@ guestfish --format=raw -a "nbd://?socket=$sock" -m /dev/sda1 <<EOF
 EOF
 
 # The original file must not be modified.
-currmod="$(stat -c "%y" cow-base.img)"
+currmod="$($STAT -c "%y" cow-base.img)"
 
 if [ "$lastmod" != "$currmod" ]; then
     echo "$0: FAILED last modified time of base file changed"
@@ -72,8 +73,8 @@ fi
 # If we have qemu-img, try the hairy rebase operation documented
 # in the nbdkit-cow-filter manual.
 if qemu-img --version >/dev/null 2>&1; then
-    qemu-img create -f qcow2 -b nbd:unix:$sock cow-diff.qcow2
-    time qemu-img rebase -b cow-base.img cow-diff.qcow2
+    qemu-img create -F raw -b nbd:unix:$sock -f qcow2 cow-diff.qcow2
+    time qemu-img rebase -F raw -b cow-base.img -f qcow2 cow-diff.qcow2
     qemu-img info cow-diff.qcow2
 
     # This checks the file we created exists.
