@@ -36,13 +36,13 @@ set -x
 
 requires_single_mode
 requires socat -h
-requires qemu-img --version
+requires nbdinfo --version
 
 sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="$sock single.disk"
 rm -f $files
 
-truncate -s 1G single.disk
+$TRUNCATE -s 1G single.disk
 
 socat unix-listen:$sock,reuseaddr,fork \
     exec:'nbdkit -r -s file single.disk' &
@@ -67,4 +67,4 @@ if ! test -S $sock; then
     exit 1
 fi
 
-qemu-img info nbd:unix:$sock
+nbdinfo "nbd+unix:///?socket=$sock"
