@@ -39,7 +39,7 @@ set -x
 
 requires qemu-img --version
 
-sock=`mktemp -u`
+sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 files="truncate3.out truncate3.pid $sock"
 rm -f $files
 cleanup_fn rm -f $files
@@ -47,11 +47,11 @@ cleanup_fn rm -f $files
 # Run nbdkit with pattern plugin and truncate filter in front.
 start_nbdkit -P truncate3.pid -U $sock \
        --filter=truncate \
-       pattern size=5G \
+       pattern 5G \
        round-up=512
 
-LANG=C qemu-img info nbd:unix:$sock > truncate3.out
-if ! grep "virtual size: 5.0G" truncate3.out; then
+qemu-img info --output=json nbd:unix:$sock > truncate3.out
+if ! grep '"virtual-size": *5368709120' truncate3.out; then
     echo "$0: unexpected output from truncate3 regression test:"
     cat truncate3.out
     exit 1
